/* Ikon MS-DOS driver test program - version 1.01 Jan-21-1993*/
/*    Copyright Interface Consultants Houston, Texas 1992,1993*/
/*    713/531-8534 713/531-8556(fax)                     */

/*
 1.01 01/21/93 - update to go with v1.01 driver

 0.92 12/04/92 - changes to go with driver v0.92
                 info ah=0x99 (was 0x0b)
                 added print mode dma (ah=11)
                 get lpt_port from ikontsr

 0.91 11/19/92 - added check for version
                   (and ikontsr loaded and initialized) */

/* this test program outputs 'hello' in print mode, and then
   does dma plot mode of blocks filled with 0xf0. There are
   100 blocks sent. The first block is 1000 bytes, and each
   block is increased in size by 1000 bytes until it rolls
   over the 16 bit max. */

/* written for Borland C (Turbo C) */

#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <alloc.h>
#include <bios.h>

#include "utypes.h"

uint lpt_port;
/*************************************************************************/
uchar print_intr(
uint bp,
uint di,
uint si,
uint ds,
uint es,
uint dx,
uint cx,
uint bx,
uint ax)
{
union REGS regs;
struct SREGS seg;

    regs.x.di = di;
    regs.x.si = si;
    seg.ds = ds;
    seg.es = es;
    regs.x.dx = dx;
    regs.x.cx = cx;
    regs.x.bx = bx;
    regs.x.ax = ax;
    int86x(0x17,&regs, &regs,&seg); /* int 0x17 is std printer svc */
    return(regs.h.ah); /* current status returned in ah */
}
/*************************************************************************/
void check_version (void) {

    /* ikontsr extended function structure */
    struct ext_fcn {
        int port;
        int fcn;
        int parm1;
        int parm2;
        int parm3;
        int parm4;
        int parm5;
        int parm6;
        int parm7;
        int parm8;
        int parm9;
        int parm10;
    } x_fcn;

    uchar rc;
    int vers;
    int err_flag = 0;

    /* extended function - ah=0x99, ds:dx points to ext_fcn structure */
    uint bp = 0;
    uint di = 0;
    uint si = 0;
    uint ds = FP_SEG(&x_fcn);
    uint es = 0;
    uint dx = FP_OFF(&x_fcn);
    uint cx = 0;
    uint bx = 0;
    uint ax = 0x9900;

        x_fcn.port  = 0;
        x_fcn.fcn   = 2;     /* get driver info */
        x_fcn.parm1 = -1; /* lpt_num is returned */
        x_fcn.parm2 = -1; /* ioaddr is returned */
        x_fcn.parm3 = -1; /* dma_mode is returned */
        x_fcn.parm4 = -1; /* major rev # is returned (0) */
        x_fcn.parm5 = -1; /* minor rev # is returned (92) */
        x_fcn.parm6 = -1; /* returns demo check */

        rc = print_intr(bp,di,si,ds,es,dx,cx,bx,ax);

        if(rc == 0x02) { /* no driver loaded */
            printf("Ikon tsr driver NOT loaded: run ikontsr and ikoninit\n");
            exit(-1);
        }

        if(x_fcn.parm1 == -1) { /* lpt_port - no status returned */
            printf("Ikon tsr driver NOT loaded: run ikontsr and ikoninit\n");
            exit(-1);
        }
        if(x_fcn.parm1 == 0xff) { /* tsr not initialized */
            printf("Ikon driver NOT initialized: run ikoninit\n");
            exit(-1);
        }

        lpt_port = x_fcn.parm1 + 1; /* set lpt port # */
        printf("ikontsr set to lpt%d:\n",lpt_port);

        if(x_fcn.parm2 == -1) { /* ioaddr or no status returned */
            err_flag = 1;
        }

        if(x_fcn.parm3 == -1) { /* dma_mode or no status returned */
            err_flag = 1;
        }

        if(x_fcn.parm4 == -1) { /* major version # or no status returned */
            err_flag = 1;
        } else {
            if(x_fcn.parm5 == -1) { /* minor version # or no status returned */
                  err_flag = 1;
            } else {
                vers = (x_fcn.parm4 * 100) + x_fcn.parm5;
                if (vers < 101) {
                    err_flag = 1;
                }
            }
        }


        if (err_flag != 0) {
           printf("Error - ikontsr version 0.93 or higher required\n");
           exit(-1);
        }
}
/*************************************************************************/
void wait_dma_complete (int lpt_port) {

    uint bp = 0;
    uint di = 0;
    uint si = 0;
    uint ds = 0;
    uint es = 0;
    uint dx = lpt_port-1;
    uint cx = 0;
    uint bx = 0;
    uint ax = 0x0400; /* wait dma complete */

    print_intr(bp,di,si,ds,es,dx,cx,bx,ax);

}
/*************************************************************************/
uchar test_print (int lpt_port,char far *string) {

    uint rc;

    /* note: you can also use function ah=11 to do print mode dma */
    while (*string != 0) {
        rc = biosprint(0,*string,(lpt_port-1));
        string++;
    }
    return(rc);
}
/*************************************************************************/
void test_plot (int lpt_port,char far *string,uint byte_count) {

    ulong phy_addr;
    uint offset;
    uchar page;
    uint word_count;
    uint bp;
    uint di;
    uint si;
    uint ds;
    uint es;
    uint dx;
    uint cx; /* word count */
    uint bx; /* offset */
    uint ax; /* dma xfer + dma page reg */


     /* get physical address */
     phy_addr = ((ulong)FP_SEG(string) <<4) + ((ulong)FP_OFF(string));
     offset = (uint)((phy_addr >> 1) & 0xffffL);
     page = (uchar)((phy_addr >>16) & 0x7eL);
     /* get the count */
     word_count = (uint)(byte_count >>1); /* word count */

    /*****************************************/
    /* start DMA xfer */
    /*****************************************/
    bp = 0;
    di = 0;
    si = 0;
    ds = 0;
    es = 0;
    dx = lpt_port-1;
    cx = word_count; /* word count */
    bx = offset; /* offset */
    ax = 0x0300 | page; /* dma xfer + dma page reg */

    print_intr(bp,di,si,ds,es,dx,cx,bx,ax);

}
/*************************************************************************/
int main(int argc, char ** argv)
{
    uint i;
    char far *plot_data;
    uint pd_count;
    uint blk_count;

    printf("Ikon MS-DOS driver test program - version 1.01 Jan-21-1993\n");
    printf("    Copyright Interface Consultants 1992,1993\n");
    /*****************************************************************/

    check_version();

    test_print(lpt_port,"hello \r\n");
    pd_count = 0xf000;
    blk_count = 1000;
    plot_data = (char far *)farmalloc(pd_count);
    if (plot_data == NULL) {
        printf("cannot allocate test buffer of %u bytes\n",pd_count);
        return(-1);
    }
    for (i=0;i<pd_count;i++) {
        plot_data[i] = 0xf0;
    }
    for (i=0; i< 60;i++) {
        printf("block = %u cnt=%u\n",i,blk_count);

        test_plot(lpt_port,plot_data,blk_count);
        blk_count += 1000; /* note: this will roll over after 65 or so */
    }
    return 0;
}


